"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NomnomlPlugin_1;
Object.defineProperty(exports, "__esModule", { value: true });
const converter_1 = require("typedoc/dist/lib/converter");
const components_1 = require("typedoc/dist/lib/converter/components");
const events_1 = require("typedoc/dist/lib/output/events");
const utils_1 = require("typedoc/dist/lib/utils");
const declaration_1 = require("typedoc/dist/lib/utils/options/declaration");
/**
 * Nomnoml plugin component.
 */
let NomnomlPlugin = NomnomlPlugin_1 = class NomnomlPlugin extends components_1.ConverterComponent {
    constructor() {
        super(...arguments);
        this.graphMap = new Map();
        /**
         * Regex literal that matches body closing tag.
         */
        this.BODY_CLOSINNG_TAG = /<\/body>/;
    }
    /**
     * 1. Load nomnoml liblary.
     * 2. Initialize nomnoml.
     * 3. Close body tag.
     */
    get customScriptsAndBodyClosinngTag() {
        const nomnomlVersion = this.nomnomlVersion;
        const script = Array.from(this.graphMap.entries())
            .map(([id, source]) => `if (document.getElementById("${id}")) {
            nomnoml.draw(document.getElementById("${id}"), ${JSON.stringify(source)});
          }`)
            .join('\n');
        return `
      <script src="https://cdnjs.cloudflare.com/ajax/libs/dagre/0.8.4/dagre.min.js"></script>
      <script src="https://cdnjs.cloudflare.com/ajax/libs/nomnoml/${nomnomlVersion}/nomnoml.min.js"></script>
      <script>${script}</script>
      </body>
    `;
    }
    /**
     * fillter logic for Comment exist
     */
    static filterComment(comment) {
        return comment !== undefined && !!comment;
    }
    /**
     * fillter logic for CommentTags exist
     */
    static filterCommentTags(tags) {
        return tags !== undefined && !!tags;
    }
    /**
     * return turn when tag's paramName is 'nomnoml'
     */
    static isNomnomlCommentTag(tag) {
        return tag.tagName === 'nomnoml';
    }
    /**
     * get CommentTags for using `@nomnoml` anotation from Context.
     */
    static nomnomlTags(context) {
        return (Object
            // get reflection from context
            .values(context.project.reflections)
            // get Comment from Reflection
            .map(reflection => reflection.comment)
            // filter only comment exist
            .filter(this.filterComment)
            // get CommentTags from Comment
            .map(comment => comment.tags)
            // filter only CommentTags exist
            .filter(this.filterCommentTags)
            // merge all CommentTags
            .reduce((a, b) => a.concat(b), [])
            // filter tag that paramName is 'nomnoml'
            .filter(this.isNomnomlCommentTag));
    }
    /**
     * The first line of text wraps h4.
     * The other wraps by div classed nomnoml.
     */
    convertCommentTagText(tagText) {
        const texts = tagText.split('\n');
        // take first line
        const title = texts.shift();
        // the other
        const nomnoml = texts.join('\n');
        const id = `nomnoml-${++NomnomlPlugin_1.graphIdCounter}`;
        this.graphMap.set(id, nomnoml);
        return `#### ${title} \n\n <canvas id="${id}"></canvas>`;
    }
    /**
     * Insert custom script before closing body tag.
     */
    convertPageContents(contents) {
        if (this.BODY_CLOSINNG_TAG.test(contents)) {
            return contents.replace(this.BODY_CLOSINNG_TAG, this.customScriptsAndBodyClosinngTag);
        }
        return contents;
    }
    /**
     * listen to event on initialisation
     */
    initialize() {
        this.listenTo(this.owner, {
            [converter_1.Converter.EVENT_RESOLVE_BEGIN]: this.onResolveBegin,
        }).listenTo(this.application.renderer, {
            [events_1.PageEvent.END]: this.onPageEnd,
        });
    }
    /**
     * Triggered when the converter begins converting a project.
     */
    onResolveBegin(context) {
        NomnomlPlugin_1.nomnomlTags(context).forEach(tag => {
            // convert
            tag.text = this.convertCommentTagText(tag.text);
        });
    }
    /**
     * Triggered after a document has been rendered, just before it is written to disc.
     * Remove duplicate lines to tidy up output
     */
    onPageEnd(page) {
        if (page.contents !== undefined) {
            // convert
            page.contents = this.convertPageContents(page.contents);
        }
    }
};
NomnomlPlugin.graphIdCounter = 0;
__decorate([
    utils_1.Option({
        name: 'nomnomlVersion',
        defaultValue: '0.6.1',
        help: 'Nomnoml Plugin: Version of nomnoml.',
        type: declaration_1.ParameterType.String,
    })
], NomnomlPlugin.prototype, "nomnomlVersion", void 0);
NomnomlPlugin = NomnomlPlugin_1 = __decorate([
    components_1.Component({ name: 'nomnoml' })
], NomnomlPlugin);
exports.NomnomlPlugin = NomnomlPlugin;
